<?php

namespace Drupal\Tests\fluent\Functional;

use Drupal\Core\Config\FileStorage;
use Drupal\file\Entity\File;
use Drupal\media\Entity\Media;
use Drupal\node\Entity\Node;
use Drupal\taxonomy\Entity\Term;
use Drupal\Tests\BrowserTestBase;

/**
 * Base functions for Fluent testing.
 *
 * @group fluent
 */
abstract class FluentTestBase extends BrowserTestBase {

  protected $defaultTheme = 'classy';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    // Modules for core functionality.
    'media',
    'node',
    'block_content',
    'comment',
    'datetime',
    'paragraphs',
    'link',

    // This custom module.
    'fluent_test',
    'fluent',
  ];

  /**
   *
   */
  protected function getMediaImage(): Media {
    $media = Media::create([
      'bundle' => 'image',
      'uid' => \Drupal::currentUser()->id(),
      'field_media_image' => [
        'target_id' => $this->getFileImage()->id(),
      ],
    ]);

    $media->save();
    return $media;
  }

  /**
   *
   */
  public function getTaxonomyTerm(): Term {
    return Term::create([
      'name' => $this->randomGenerator->string(),
      'vid' => 'tags',
    ]);
  }

  /**
   *
   */
  protected function getFileImage(): File {
    $file_data = file_get_contents(
      $this->randomGenerator->image('/tmp/rand.jpg', '200x200', '500x500')
    );
    $fileRepo = \Drupal::service('file.repository');

    return $fileRepo->writeData($file_data, 'public://' . $this->randomGenerator->string() . '/');
  }

  protected function getFloat($min = 0, $max = 1, $n = 2) {
    return strval(round($min + lcg_value() * (abs($max - $min)), $n));
  }

  protected function getCurrentDate($format, $time) {
    if ($format === 'date') {
      $currentDate = date('Y-m-d', $time);
    } else if ($format === 'date_time') {
      $currentDate = date('Y-m-d', time()) . 'T' . date('H:i:s', $time);
    }

    return $currentDate;
  }

  /**
   *
   */
  protected function getNode(): Node {
    $node = Node::create([
      'type' => 'content_fields',
      'title' => $this->randomGenerator->string(),
      'field_boolean_single' => strval(rand(0, 1)),
      'field_color_multi' => [
        0 => ['color' => '#F7B32B', 'opacity' => $this->getFloat()],
        1 => ['color' => '#AB87FF', 'opacity' => $this->getFloat()],
      ],
      'field_color_single' => ['color' => '#F48668', 'opacity' => $this->getFloat()],
      'field_date_date_single' => ["value" => $this->getCurrentDate('date', time())],
      'field_date_date_time_single' => ["value" => $this->getCurrentDate('date_time', time())],
      'field_date_range_all_day_single' => [
        "value" => $this->getCurrentDate('date_time', time()),
        "end_value" => $this->getCurrentDate('date_time', time() + 1),
      ],
      'field_date_range_date_single' => [
        "value" => $this->getCurrentDate('date', time()),
        "end_value" => $this->getCurrentDate('date', time() + 86400),
      ],
      'field_date_range_datetime_single' => [
        "value" => $this->getCurrentDate('date_time', time()),
        "end_value" => $this->getCurrentDate('date_time', time() + 1),
      ],
    ]);

    $node->save();
    return $node;
  }

}
