<?php

namespace Drupal\Tests\fluent\Functional;

use Drupal\Core\Datetime\DrupalDateTime;

/**
 * Test basic functionality of My Module.
 *
 * @group fluent
 */
class SingleValueTestCase extends FluentTestBase {

  public function testBaseNodeField() {
    $node = $this->getNode();
    $title = $node->getTitle();

    $this->assertEquals($title, using($node)->value('title'));
    $this->assertEquals('en', using($node)->value('langcode'));

    $this->assertIsInt(using($node)->value('nid'));
    $this->assertIsString(using($node)->value('id'));
    $this->assertInstanceOf(DrupalDateTime::class, using($node)->value('changed'));
    $this->assertInstanceOf(DrupalDateTime::class, using($node)->value('created'));
  }

  public function testCreatedField() {
    $image = $this->getMediaImage();
    $timestamp = $image->getCreatedTime();
    $ddateTime = fluent()->using($image)->value('created');

    $this->assertInstanceOf(DrupalDateTime::class, $ddateTime);
    $this->assertEquals($ddateTime->getTimestamp(), $timestamp);
  }

  public function testTaxonomyTermField() {
    // @todo
  }

  /**
   * Test field_boolean_single
   */
  public function testFieldBooleanSingle(){
    $node = $this->getNode();
    $boolean_single = $node->get('field_boolean_single')->getString() === '1' ? true : false;

    $this->assertSame($boolean_single, using($node)->value('field_boolean_single'));
  }

  /**
   * Test field_color_single
   */
  public function testFieldColorSingle(){
    $node = $this->getNode();
    $color = $node->get('field_color_single')->getValue()[0]['color'];
    $opacity = (float)$node->get('field_color_single')->getValue()[0]['opacity'];

    $this->assertSame($color, using($node)->value('field_color_single.color'));
    $this->assertSame($opacity, using($node)->value('field_color_single.opacity'));
  }

  /**
   * Test field_date_time_single
   */
  public function testFieldDateTimeSingle() {
    $node = $this->getNode();
    $date_fields = [
      'date' => 'field_date_date_single',
      'date_time' => 'field_date_date_time_single',
      'start_date' => 'field_date_range_all_day_single.value',
      'end_date' => 'field_date_range_all_day_single.end_value',
    ];
    $date_range_fields = [
      'field_date_range_all_day_single',
      'field_date_range_date_single',
      'field_date_range_datetime_single',
    ];

    foreach ($date_fields as $key => $field) {
      $fluent_value = using($node)->value($field);

      if ($key === 'date' || $key === 'date_time') {
        $drupal_value = $node->get($field)->date;
      } else if ($key === 'start_date' || $key === 'end_date') {
        $drupal_value = $node->get($date_range_fields[0])->$key;
      }

      $this->assertInstanceOf(DrupalDateTime::class, $fluent_value);  //Is DrupalDateTime instance
      $this->assertEquals($drupal_value, $fluent_value);  //Value comparation
    }

    foreach ($date_range_fields as $field) {
      $fluent_value = using($node)->value($field);
      $drupal_value = [
        'start_date' => $node->get($field)->start_date,
        'end_date' => $node->get($field)->end_date,
      ];

      $this->assertIsArray($fluent_value);  //Is Array
      $this->assertSame($drupal_value, $fluent_value);  //Value comparation
    }

  }

}
